@@ -556,6 +556,9 @@ int gl_get_count(GLenum param)
 	case GL_UNPACK_SWAP_BYTES:
 	case GL_ZOOM_X:
 	case GL_ZOOM_Y:
+#ifdef GL_VERSION_3_0
+	case GL_MAX_ARRAY_TEXTURE_LAYERS:
+#endif
 		return 1;
 	case GL_DEPTH_RANGE:
 	case GL_LINE_WIDTH_RANGE:
@@ -955,33 +958,33 @@ void pgl_set_type(SV * sv, GLenum type, void ** ptr)
 	{
 		unsigned long v = SvIV(sv);
 		(*(GLubyte*)*ptr) = (GLubyte)(v >> 8);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		(*(GLubyte*)*ptr) = (GLubyte)(v & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		break;
 	}
 	case GL_3_BYTES:
 	{
 		unsigned long v = SvIV(sv);
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 16) & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 8) & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 0) & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		break;
 	}
 	case GL_4_BYTES:
 	{
 		unsigned long v = SvIV(sv);
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 24) & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 16) & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 8) & 0xff);
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		(*(GLubyte*)*ptr) = (GLubyte)((v >> 0) & 0xff);
-		*ptr++;
+		(*ptr)++;
 		break;
 	}
 	default:
@@ -1039,9 +1042,9 @@ SV * pgl_get_type(GLenum type, void ** ptr)
 	{
 		unsigned long v;
 		v = (*(GLubyte*)*ptr) << 8;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		v |= (*(GLubyte*)*ptr) << 0;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		result = newSViv(v);
 		break;
 	}
@@ -1049,11 +1052,11 @@ SV * pgl_get_type(GLenum type, void ** ptr)
 	{
 		unsigned long v;
 		v = (*(GLubyte*)*ptr) << 16;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		v |= (*(GLubyte*)*ptr) << 8;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		v |= (*(GLubyte*)*ptr) << 0;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		result = newSViv(v);
 		break;
 	}
@@ -1061,13 +1064,13 @@ SV * pgl_get_type(GLenum type, void ** ptr)
 	{
 		unsigned long v;
 		v = (*(GLubyte*)*ptr) << 24;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		v |= (*(GLubyte*)*ptr) << 16;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		v |= (*(GLubyte*)*ptr) << 8;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		v |= (*(GLubyte*)*ptr) << 0;
-		*(unsigned char**)ptr++;
+		(*(unsigned char**)ptr)++;
 		result = newSViv(v);
 		break;
 	}
@@ -1176,312 +1179,3 @@ GLsizei width, GLsizei height, GLsizei depth, GLenum format, GLenum type, int mo
 }
 
 
-#ifdef __PM__
-
-/* A very primitive emulation level for X under PM... */
-
-#  include "os2pm_X.h"
-
-int yScreen;		/* Will update ASAP */
-
-void
-InitSys(void) {
-    yScreen = WinQuerySysValue(HWND_DESKTOP, SV_CYSCREEN);
-    EventAv = newAV();
-}
-
-Bool
-XQueryPointer(display, w, root_return, child_return, root_x_return,
-	root_y_return, win_x_return, win_y_return, mask_return)
-    Display* display;
-    Window w;
-    Window* root_return;
-    Window* child_return;
-    int* root_x_return;
-    int* root_y_return;
-    int* win_x_return;
-    int* win_y_return;
-    unsigned int* mask_return;
-{
-    POINTL pos;
-    unsigned int  state = 0;
-    static SWP clientsize;
-
-#ifdef XDEBUG
-    printf("XQueryPointer\n");
-#endif
-    WinQueryPointerPos(HWND_DESKTOP, &pos);
-    *root_x_return = pos.x;
-    /* Translate from PM to X coordinates */
-    *root_y_return = yScreen - pos.y;
-
-    WinMapWindowPoints (HWND_DESKTOP, w, &pos, 1);
-    *win_x_return = pos.x;
-    /* Translate from PM to X coordinates */
-    WinQueryWindowPos(w,&clientsize);
-    *win_y_return = clientsize.cy - pos.y;
-
-    *root_return = HWND_DESKTOP;
-    *child_return = w;
-
-    if (WinGetKeyState(HWND_DESKTOP, VK_BUTTON1) & 0x8000)
-	state |= Button1MaskOS2;
-    /* Report mouse as with X */
-    if (WinGetKeyState(HWND_DESKTOP, VK_BUTTON2) & 0x8000)
-	state |= Button2MaskOS2;
-    if (WinGetKeyState(HWND_DESKTOP, VK_BUTTON3) & 0x8000)
-	state |= Button3MaskOS2;
-    if (WinGetKeyState(HWND_DESKTOP, VK_CTRL) & 0x8000)
-	state |= ControlMask;
-    if (WinGetKeyState(HWND_DESKTOP, VK_SHIFT) & 0x8000)
-	state |= ShiftMask;
-    if (WinGetKeyState(HWND_DESKTOP, VK_ALT) & 0x8000)
-	state |= Mod4Mask;
-    if (WinGetKeyState(HWND_DESKTOP, VK_CAPSLOCK) & 0x0001)
-	state |= LockMask;
-    if (WinGetKeyState(HWND_DESKTOP, VK_NUMLOCK) & 0x0001)
-	state |= Mod1Mask;
-    if (WinGetKeyState(HWND_DESKTOP, VK_SCRLLOCK) & 0x0001)
-	state |= Mod3Mask;
-    *mask_return = state;
-#ifdef XDEBUG
-    printf("sx = %d, sy = %d, wx = %d, wx = %d, mask = %#x. \n",
-	*root_x_return, *root_y_return, *win_x_return, *win_y_return, *mask_return);
-#endif
-    return True;
-}
-
-void
-XNextEvent(Display *d, XEvent *event)
-{
-  SV* sv = av_shift(EventAv);
-  IV n_a;
-
-  StructCopy((XEvent*)SvPV(sv, n_a),event, XEvent);
-  SvREFCNT_dec(sv);
-}
-
-void
-XLookupString(XKeyEvent *xkey, char *buf, int sizeof_buf, KeySym *ks, int f)
-{
-  if (xkey->keycode < 256)
-    *ks = (KeySym)xkey->keycode;
-  else if (xkey->keycode == 256 + VK_ESC)
-    *ks = (KeySym)27;
-  else if (xkey->keycode == 256 + VK_ENTER)
-    *ks = (KeySym)'\n';
-  else if (xkey->keycode == 256 + VK_SPACE)
-    *ks = (KeySym)' ';
-  else if (xkey->keycode == 256 + VK_TAB)
-    *ks = (KeySym)'\t';
-  else
-    *ks = (KeySym)'\0';
-}
-
-
-static MRESULT EXPENTRY WindowProc(HWND hwnd, ULONG msg, MPARAM mp1, MPARAM
-mp2)
-{
-  static float t = 0.0;
-  static SWP clientsize;
-  static USHORT mycode;
-  static ULONG key;
-
-  switch(msg) {
-  case WM_SIZE:
-    WinQueryWindowPos(hwnd,&clientsize);
-    if (LastEventMask & StructureNotifyMask) {
-      XEvent x;
-      SV *ev;
-
-      x.type = ConfigureNotify;
-      x.xconfigure.width = clientsize.cx;
-      x.xconfigure.height = clientsize.cy;
-      av_push(EventAv, newSVpv((char*)&x,sizeof(XEvent)));
-    } else {
-      /* Upon a resize, query new window size and set OpenGL viewport */
-      glViewport(0, 0, clientsize.cx, clientsize.cy);
-    }
-    return WinDefWindowProc(hwnd, msg, mp1, mp2);
-  case WM_TIMER:
-    /* Upon getting a timer message, the invalidate rectangle call  */
-    /* will cause a WM_PAINT message to be sent, enabling animation */
-    WinInvalidateRect(hwnd, NULLHANDLE, 0);
-    return WinDefWindowProc(hwnd, msg, mp1, mp2);
-  case WM_PAINT:
-    if (LastEventMask & ExposureMask) {
-      XEvent x;
-      SV *ev;
-
-      x.type = Expose;
-      av_push(EventAv, newSVpv((char*)&x,sizeof(XEvent)));
-    }
-    return WinDefWindowProc(hwnd, msg, mp1, mp2);
-  case WM_CHAR:
-    mycode = (USHORT)SHORT1FROMMP(mp1);
-    if ((mycode & KC_CHAR) && !(mycode & KC_KEYUP))
-      key = CHAR1FROMMP(mp2);
-    else if ((mycode & KC_VIRTUALKEY) && !(mycode & KC_KEYUP))
-      key = CHAR3FROMMP(mp2) + 256;
-    else
-      key = 0;
-    if (key && (LastEventMask & KeyPressMask)) {
-      XEvent x;
-      SV *ev;
-
-      x.type = KeyPress;
-      x.xkey.keycode = key;
-      av_push(EventAv, newSVpv((char*)&x,sizeof(XEvent)));
-    }
-    return WinDefWindowProc(hwnd, msg, mp1, mp2);
-  default:
-    return WinDefWindowProc(hwnd, msg, mp1, mp2);
-  }
-}
-
-struct Tk_Window_t {long type; HWND w;};
-
-Window
-nativeWindowId(Display *d, Window id)
-{
-    if (!WinIsWindow(*d,id))			 /* Tk handle? */
-	return ((struct Tk_Window_t*)id)->w;
-    return id;
-}
-
-
-Window
-MyCreateWindow(Display *d, Window par, int x, int y, int w, int h)
-{
-    HWND hwnd = 0, hwndTop;
-    int err;
-
-/*    fprintf(stderr, "Creating with parent=%#lx.\n", (long)par);	*/
-    if (par != HWND_DESKTOP) {
-      ULONG createflags = 0;
-
-      par = nativeWindowId(d, par);
-/*    fprintf(stderr, "Creating with parent=%#lx.\n", (long)par);	*/
-      if (!WinRegisterClass( Perl_hab,
-                             (PSZ)"PerlGLkid",
-                             WindowProc,
-                             CS_MOVENOTIFY, /* Need at least this! */
-			     /* As documented CS_SIZEREDRAW is needed too... */
-			     /* But this would not improve the visual appearence... */
-                             0))
-          croak("Cannot register class");
-      err = CheckWinError(
-		hwnd = WinCreateWindow(
-                        par,   		/* Parent    */
-                        (PSZ)"PerlGLkid", /* class name              */
-                        NULL,		/* window title            */
-                        WS_VISIBLE,     /* Window style             */
-			x,y,w,h,
-                        par,   		/* Owner    */
-                        HWND_TOP,	/* Position    */
-                        FID_CLIENT,     /* Standard id         */
-                        0,              /* No class data         */
-                        0));            /* No presentation parameters */
-    } else {
-      ULONG createflags = FCF_TITLEBAR |
-                        FCF_SYSMENU    |
-                        FCF_MINMAX     |
-                        FCF_TASKLIST   |
-                        FCF_SIZEBORDER;
-    
-      if (!WinRegisterClass( Perl_hab,
-                             (PSZ)"PerlGLtop",
-                             WindowProc,
-                             CS_SIZEREDRAW | CS_MOVENOTIFY, /* Need at least this! */
-                             0))
-          croak("Cannot register class");
-      err = CheckWinError(
-	      hwndTop = WinCreateStdWindow(
-                        par,   /* Parent    */
-                        WS_VISIBLE,     /* Frame style             */
-                        &createflags,   /* min FCF_MENU|FCF_MINMAX */
-                        (PSZ)"PerlGLtop", /* class name              */
-                        "OpenGL Sample",/* window title            */
-                        WS_VISIBLE,     /* client style            */
-                        0,              /* resource handle         */
-                        1,              /* Resource ID             */
-                        &hwnd));
-    }
-    if (err)
-	croak("Cannot create window, $^E=%#x", WinGetLastError(*d));
-    if (!hwnd)
-	croak("Error creating window");
-        /* you must set window size before you call pglMakeCurrent */
-    if ((par == HWND_DESKTOP) &&
-	!WinSetWindowPos( hwndTop,
-			  HWND_TOP,
-			  x,
-			  y,
-			  w,		/* XXXX Add border! */
-			  h,
-			  SWP_ACTIVATE | SWP_SIZE | SWP_MOVE | SWP_SHOW))
-          croak("Couldn't position window!\n");
-    return hwnd;
-}
-
-void
-glpMoveResizeWindow(int x, int y, unsigned int width, unsigned int height, Window w, Display* display)
-{
-    SWP parPos;
-
-    w = nativeWindowId(display, w);
-    WinQueryWindowPos(WinQueryWindow(w, QW_PARENT), &parPos);
-    /* Translate Y coordinates to PM: relative to parent */
-    WinSetWindowPos(w, HWND_TOP, x, parPos.cy - height - y,
-                    width, height, SWP_MOVE | SWP_SIZE);
-}
-
-void
-glpMoveWindow(int x, int y, Window w, Display* display)
-{
-    SWP parPos;
-    SWP myPos;
-
-    w = nativeWindowId(display, w);
-    WinQueryWindowPos(WinQueryWindow(w, QW_PARENT), &parPos);
-    WinQueryWindowPos(w, &myPos);
-    /* Translate Y coordinates to PM: relative to parent */
-    WinSetWindowPos(w, HWND_TOP, x, parPos.cy - myPos.cy - y,
-                    0, 0, SWP_MOVE);
-}
-
-void
-glpResizeWindow(unsigned int width, unsigned int height, Window w, Display* display)
-{
-    SWP parPos;
-    SWP myPos;
-
-    w = nativeWindowId(display, w);
-    WinQueryWindowPos(WinQueryWindow(w, QW_PARENT), &parPos);
-    WinQueryWindowPos(w, &myPos);
-    /* Translate Y coordinates to PM: relative to parent... */
-    /* Need to move too to leave the upper left corner at the same place... */
-    WinSetWindowPos(w, HWND_TOP, myPos.x, parPos.cy - myPos.cy - myPos.y,
-		    width, height, SWP_MOVE | SWP_SIZE);
-}
-
-void
-morphPM()
-{
-    PPIB pib;
-    PTIB tib;
-
-    DosGetInfoBlocks(&tib, &pib);
-    if (pib->pib_ultype != 3)		/* 2 is VIO */
-	pib->pib_ultype = 3;		/* 3 is PM */
-}
-
-#undef glutCreateWindow
-int
-my_glutCreateWindow(name)
-{
-    morphPM();
-    return glutCreateWindow(name);
-}
-
-#endif
